/*****************************************************************/
/* TOOLS                                                 V.Burel */
/*****************************************************************/
/* General Functions to be used in MinPrg Samples                */
/*****************************************************************/
#ifndef __cplusplus
	#ifndef STRICT
		#define STRICT
	#endif
#endif

#include <windows.h>
#include <stdio.h>
#include "tools.h"

//Make this following define if your compiler does 
//not support microsoft inline assembly language:

//#define MY_COMPILER_DOESNOT_SUPPORT_MSASM

/*****************************************************************/
/* Manage Application Directory                                  */
/*****************************************************************/

static char G_CurrentDirectory[1024];

void TOOL_StoreOriginalAppDirectory(void)
{
	GetCurrentDirectory(1020,G_CurrentDirectory);
}

void TOOL_GetAppDirectory(char *  dir)
{
	if (dir != NULL) strcpy(dir,G_CurrentDirectory);
}

void TOOL_GetPathOnly(char *  sss)
{
	long ll;
	ll=(long)strlen(sss);
	while ((ll>0) && (sss[ll]!='\\')) ll--;
	if (sss[ll] == '\\') sss[ll]=0;
}

/*****************************************************************/
/* Memory Block Functions                                        */
/*****************************************************************/

// Initialize a memory block with ZERO 

void __cdecl TOOL_MemZERO(void * target, long nbByte)
{
#ifdef MY_COMPILER_DOESNOT_SUPPORT_MSASM
	memset(target,0,nbByte);
#else
	__asm {
		push edi
		push esi
		mov eax,DWORD PTR nbByte;
		mov edi,DWORD PTR target;
		xor edx,edx
		cld
		mov ecx,eax
		shr ecx,3
		jz SHORT P_MZ_next1
	P_MZ_loop1:
		mov DWORD PTR [edi],edx
		mov DWORD PTR [edi+0x00000004],edx
		add edi,0x00000008
		dec ecx
		jnz SHORT P_MZ_loop1
	P_MZ_next1:
		and eax,0x00000007
		jz SHORT P_MZ_end1		
		mov ecx,eax
		xor eax,eax
		rep stosb
	P_MZ_end1:
		pop esi
		pop edi
	}
#endif
}



/*****************************************************************/
/* Bitmap Functions                                              */
/*****************************************************************/

long TOOL_GetBitmapSize(HBITMAP bmp,long * dx,long * dy)
{
	BITMAP	bmpinfo;
	if (bmp == NULL) return -1;
	GetObject(bmp,sizeof(BITMAP),&bmpinfo);
	if (dx != NULL) *dx=bmpinfo.bmWidth;
	if (dy != NULL) *dy=bmpinfo.bmHeight;
	return 0;
}

void TOOL_DrawBitmap(HDC dc, int x0, int y0, HBITMAP hbmp)
{
	HBITMAP		oldBitmap;
	BITMAP bm;
	HDC dcmem;
	dcmem = CreateCompatibleDC(dc);
	oldBitmap=(HBITMAP)SelectObject(dcmem,hbmp);
	GetObject(hbmp,sizeof(BITMAP),&bm);
	BitBlt(dc,x0,y0,bm.bmWidth,bm.bmHeight,dcmem,0,0,SRCCOPY);
	SelectObject(dcmem,oldBitmap);
	DeleteDC(dcmem);
}

void TOOL_DrawPartOfBitmap(HDC dc,HBITMAP hbmp,int x0, int y0,int bmpx0,int bmpy0, int bmpcx,int bmpcy)
{
	HBITMAP		oldBitmap;
	HDC 		bitmapDC;
	bitmapDC=CreateCompatibleDC(dc);
	oldBitmap=(HBITMAP)SelectObject(bitmapDC,hbmp);
	BitBlt(dc,x0,y0,bmpcx,bmpcy,bitmapDC,bmpx0,bmpy0,SRCCOPY);
	SelectObject(bitmapDC,oldBitmap);
	DeleteDC(bitmapDC);
}



/*****************************************************************/
/* Get Window Metrics Info                                       */
/*****************************************************************/

#ifndef SM_CXPADDEDBORDER
	#define SM_CXPADDEDBORDER 92
#endif

typedef struct tagVBNONCLIENTMETRICS 
{  
	UINT cbSize;
	int iBorderWidth;
	int iScrollWidth;
	int iScrollHeight;
	int iCaptionWidth;
	int iCaptionHeight;
	LOGFONT lfCaptionFont;
	int iSmCaptionWidth;
	int iSmCaptionHeight;
	LOGFONT lfSmCaptionFont;
	int iMenuWidth;
	int iMenuHeight;
	LOGFONT lfMenuFont;
	LOGFONT lfStatusFont;
	LOGFONT lfMessageFont;
	int iPaddedBorderWidth;
} VBNONCLIENTMETRICS,  *LPVBNONCLIENTMETRICS;


void TOOL_GetMetricInfoWindow(long * win_bordersize, long * win_captiondy,long * win_menudy)
{
	long xyadjust=0;
	BOOL ok;
	VBNONCLIENTMETRICS ncm;
	ncm.cbSize=sizeof(VBNONCLIENTMETRICS);
	//For Vista
	ok=SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(VBNONCLIENTMETRICS),&ncm,0L); 
	if (ok != 0)
	{
		xyadjust=ncm.iPaddedBorderWidth+2;
		if (win_menudy != NULL)			*win_menudy		=ncm.iMenuHeight;
		if (win_bordersize != NULL)		*win_bordersize	=xyadjust+ncm.iBorderWidth;
		if (win_captiondy != NULL)		*win_captiondy	=ncm.iCaptionHeight;
		return;
	}
	//For Windows 2000/XP
	xyadjust=GetSystemMetrics(SM_CXPADDEDBORDER);
	if (win_menudy != NULL)		*win_menudy=GetSystemMetrics(SM_CYMENU);
	if (win_bordersize != NULL) *win_bordersize= xyadjust+GetSystemMetrics(SM_CXDLGFRAME);
	if (win_captiondy != NULL)	*win_captiondy=GetSystemMetrics(SM_CYCAPTION);
}
